"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
function __export(m) {
    for (var p in m) if (!exports.hasOwnProperty(p)) exports[p] = m[p];
}
Object.defineProperty(exports, "__esModule", { value: true });
const express = require("express");
const graphql_playground_middleware_express_1 = require("graphql-playground-middleware-express");
const module_1 = require("./module");
const mink_1 = require("@zuu/mink");
const owl_1 = require("@zuu/owl");
const subscriptions_transport_ws_1 = require("subscriptions-transport-ws");
const vet_1 = require("@zuu/vet");
const ListeningEvent_1 = require("./events/ListeningEvent");
const ferret_1 = require("@zuu/ferret");
const ConnectionCreatedEvent_1 = require("./events/ConnectionCreatedEvent");
const SubscriptionServerListeningEvent_1 = require("./events/SubscriptionServerListeningEvent");
const http_1 = require("http");
const graphql_1 = require("graphql");
const GQLHelper_1 = require("./GQLHelper");
__export(require("./events"));
__export(require("./module"));
__export(require("./GQLHelper"));
class BootstrappedOptions {
    constructor(options) {
        options = options || {};
        options.listeners = options.listeners || [];
        options.server = options.server || {};
        options.server.port = options.server.port || 4000;
        options.graph = options.graph || {};
        options.resolvers = options.resolvers || [];
        options.controllers = options.controllers || [];
        options.graph.queryEndpoint = options.graph.queryEndpoint || GQLHelper_1.GQLHelper.queryPath;
        options.graph.subscriptionsEndpoint = options.graph.subscriptionsEndpoint || GQLHelper_1.GQLHelper.subscriptionsPath;
        options.graph.playground = options.graph.playground || GQLHelper_1.GQLHelper.playground;
        options.graph.contextFiller = options.graph.contextFiller || ((user, headers) => { return {}; });
        options.graph.subscriptionCurrentUserChecker = options.graph.subscriptionCurrentUserChecker;
        GQLHelper_1.GQLHelper.contextFiller = options.graph.contextFiller;
        GQLHelper_1.GQLHelper.subscriptionCurrentUserChecker = options.graph.subscriptionCurrentUserChecker;
        this._options = options;
    }
    getOptions() {
        return this._options;
    }
    run(app) {
        return __awaiter(this, void 0, void 0, function* () {
            let options = this._options;
            for (let i = 0; i < options.listeners.length; i++) {
                vet_1.EventBus.subscribe(options.listeners[i]);
            }
            if (options.resolvers.length != 0)
                yield owl_1.GQLFactory.setup(options.resolvers);
            let connection;
            if (typeof options.model != "undefined")
                connection = yield ferret_1.createConnection(options.model);
            else
                connection = yield ferret_1.createConnection();
            vet_1.EventBus.emit(new ConnectionCreatedEvent_1.ConnectionCreatedEvent(connection));
            if (!app)
                app = express();
            let modules = options.server.modules || [];
            module_1.ModuleLoader.loadMany(modules);
            module_1.ModuleLoader.before(app);
            mink_1.useExpressServer(app, options);
            module_1.ModuleLoader.after(app);
            if (typeof options.graph.playground != "undefined" && options.graph.playground != null) {
                app.get(options.graph.playground, graphql_playground_middleware_express_1.default({
                    endpoint: options.graph.queryEndpoint || "/",
                    subscriptionEndpoint: options.graph.subscriptionsEndpoint || "/"
                }));
            }
            let server = http_1.createServer(app);
            app.set("PORT", options.server.port);
            yield (() => { return new Promise($ => { server.listen(options.server.port, _ => { $(); }); }); })();
            vet_1.EventBus.emit(new ListeningEvent_1.ListeningEvent(app));
            let subscriptionServer = undefined;
            if (typeof options.graph.subscriptionsEndpoint != "undefined" && options.graph.subscriptionsEndpoint != null && typeof owl_1.GQLFactory.schema != "undefined" && owl_1.GQLFactory.schema != null) {
                let onConnect = options.graph.subscriptionCurrentUserChecker;
                subscriptionServer = new subscriptions_transport_ws_1.SubscriptionServer({ schema: owl_1.GQLFactory.schema, execute: graphql_1.execute, subscribe: graphql_1.subscribe, onConnect }, { server, path: options.graph.subscriptionsEndpoint });
                vet_1.EventBus.emit(new SubscriptionServerListeningEvent_1.SubscriptionServerListeningEvent(subscriptionServer));
            }
            return { app, server, subscriptionServer };
        });
    }
}
exports.BootstrappedOptions = BootstrappedOptions;
class Bootstrap {
    static getActiveBootstrap() {
        return this._current;
    }
    static scope(options) {
        if (!this._current)
            this._current = new BootstrappedOptions(options);
        return this._current;
    }
}
exports.Bootstrap = Bootstrap;
__export(require("./controllers"));
